@@ -22,6 +22,7 @@
  */
 
 #include "psxcounters.h"
+#include "gpu.h"
 #include "debug.h"
 
 /******************************************************************************/
@@ -70,10 +71,10 @@ static const s32 VerboseLevel     = VERBOSE_LEVEL;
 
 Rcnt rcnts[ CounterQuantity ];
 
-static u32 hSyncCount = 0;
+u32 hSyncCount = 0;
+u32 frame_counter = 0;
 static u32 spuSyncCount = 0;
 static u32 hsync_steps = 0;
-static u32 gpu_wants_hcnt = 0;
 static u32 base_cycle = 0;
 
 u32 psxNextCounter = 0, psxNextsCounter = 0;
@@ -189,20 +190,16 @@ void psxRcntReset( u32 index )
 {
     u32 count;
 
+    rcnts[index].mode |= RcUnknown10;
+
     if( rcnts[index].counterState == CountToTarget )
     {
+        count  = psxRegs.cycle;
+        count -= rcnts[index].cycleStart;
+        if( rcnts[index].rate > 1 )
+            count /= rcnts[index].rate;
         if( rcnts[index].mode & RcCountToTarget )
-        {
-            count  = psxRegs.cycle;
-            count -= rcnts[index].cycleStart;
-            if (rcnts[index].rate > 1)
-                count /= rcnts[index].rate;
             count -= rcnts[index].target;
-        }
-        else
-        {
-            count = _psxRcntRcount( index );
-        }
 
         _psxRcntWcount( index, count );
 
@@ -217,8 +214,12 @@ void psxRcntReset( u32 index )
         }
 
         rcnts[index].mode |= RcCountEqTarget;
+
+        if( count < 0xffff ) // special case, overflow too?
+            return;
     }
-    else if( rcnts[index].counterState == CountToOverflow )
+
+    if( rcnts[index].counterState == CountToOverflow )
     {
         count  = psxRegs.cycle;
         count -= rcnts[index].cycleStart;
@@ -240,10 +241,6 @@ void psxRcntReset( u32 index )
 
         rcnts[index].mode |= RcOverflow;
     }
-
-    rcnts[index].mode |= RcUnknown10;
-
-    psxRcntSet();
 }
 
 void psxRcntUpdate()
@@ -293,22 +290,24 @@ void psxRcntUpdate()
         // VSync irq.
         if( hSyncCount == VBlankStart[Config.PsxType] )
         {
-            GPU_vBlank( 1, &hSyncCount, &gpu_wants_hcnt );
-            
-            // For the best times. :D
-            //setIrq( 0x01 );
+            if( !(HW_GPU_STATUS & PSXGPU_ILACE) )
+                HW_GPU_STATUS |= PSXGPU_LCF;
+
+            setIrq( 0x01 );
+
+            EmuUpdate();
+            GPU_updateLace();
         }
         
         // Update lace. (with InuYasha fix)
         if( hSyncCount >= (Config.VSyncWA ? HSyncTotal[Config.PsxType] / BIAS : HSyncTotal[Config.PsxType]) )
         {
             hSyncCount = 0;
+            frame_counter++;
 
-            GPU_vBlank( 0, &hSyncCount, &gpu_wants_hcnt );
-            setIrq( 0x01 );
-
-            EmuUpdate();
-            GPU_updateLace();
+            HW_GPU_STATUS &= ~PSXGPU_LCF;
+            if( HW_GPU_STATUS & PSXGPU_ILACE )
+                HW_GPU_STATUS |= frame_counter << 31;
         }
 
         // Schedule next call, in hsyncs
@@ -319,8 +318,6 @@ void psxRcntUpdate()
             hsync_steps = next_vsync;
         if( next_lace && next_lace < hsync_steps )
             hsync_steps = next_lace;
-        if( gpu_wants_hcnt )
-            hsync_steps = 1;
 
         rcnts[3].cycleStart = cycle - leftover_cycles;
         if (Config.PsxType)
@@ -331,9 +328,10 @@ void psxRcntUpdate()
                 base_cycle += hsync_steps * 8791293;
         rcnts[3].cycle = base_cycle >> 12;
         base_cycle &= 0xfff;
-        psxRcntSet();
     }
 
+    psxRcntSet();
+
 #ifndef NDEBUG
     DebugVSync();
 #endif
