@@ -57,13 +57,13 @@ window_key_proxy(GtkWidget *widget,
 		     GdkEventKey *event,
 		     gpointer user_data)
 {
-return key_press_event(keymap[event->hardware_keycode], event->type == GDK_KEY_PRESS?1:(event->type == GDK_KEY_RELEASE?2:0) );
+return key_press_event(event->hardware_keycode, event->type == GDK_KEY_PRESS?1:(event->type == GDK_KEY_RELEASE?2:0) );
 }
 
-inline void key_press_event(int key,int type)
+inline void key_press_event(int key2,int type)
 {
 	int  psxkey1 = -1, psxkey2 = -1;
-
+int key=keymap[key2];
 	
 	if (key < 0)
 		return;
@@ -158,7 +158,7 @@ void maemo_init(int *argc, char ***argv)
 		sens=150;
 		y_def=500; //near 45 degrees =)
 	}
-
+if (!(g_maemo_opts&8)){
 	gtk_init (argc, argv);
 
 	window = hildon_stackable_window_new ();
@@ -186,6 +186,7 @@ void maemo_init(int *argc, char ***argv)
 
 	gtk_widget_show_all (GTK_WIDGET (actor));
 	gtk_widget_show_all (GTK_WIDGET (window));
+	}
 }
 
 void menu_loop(void)
