@@ -25,9 +25,23 @@ int vout_finish(void)
   return 0;
 }
 
+static void check_mode_change(void)
+{
+  static uint32_t old_status;
+  static int old_h;
+
+  // width|rgb24 change?
+  if ((gpu.status.reg ^ old_status) & ((7<<16)|(1<<21)) || gpu.screen.h != old_h)
+  {
+    old_status = gpu.status.reg;
+    old_h = gpu.screen.h;
+    screen_buf = cbs->pl_vout_set_mode(gpu.screen.hres,
+                                       gpu.screen.h, gpu.status.rgb24 ? 24 : 16);
+  }
+}
+
 static void blit(void)
 {
-  static uint32_t old_status, old_h;
   int x = gpu.screen.x & ~1; // alignment needed by blitter
   int y = gpu.screen.y;
   int w = gpu.screen.w;
@@ -38,15 +52,7 @@ static void blit(void)
   uint8_t *dest;
 
   fb_offs = y * 1024 + x;
-
-  if ((gpu.status.reg ^ old_status) & ((7<<16)|(1<<21)) || h != old_h) // width|rgb24 change?
-  {
-    old_status = gpu.status.reg;
-    old_h = h;
-    screen_buf = cbs->pl_vout_set_mode(stride, h, gpu.status.rgb24 ? 24 : 16);
-  }
-
-  dest = screen_buf;
+  dest = (uint8_t *)screen_buf;
 
   // only do centering, at least for now
   doffs = (stride - w) / 2 & ~1;
@@ -88,20 +94,23 @@ void GPUupdateLace(void)
     return;
 
   if (gpu.frameskip.set) {
-    if (!gpu.frameskip.frame_ready && gpu.frameskip.skipped_blits < 9) {
-      gpu.frameskip.skipped_blits++;
-      return;
+    if (!gpu.frameskip.frame_ready) {
+      if (*gpu.state.frame_count - gpu.frameskip.last_flip_frame < 9)
+        return;
     }
     gpu.frameskip.frame_ready = 0;
-    gpu.frameskip.skipped_blits = 0;
   }
 
   renderer_flush_queues();
-  blit();
+  check_mode_change();
+  if (cbs->pl_vout_raw_flip)
+    cbs->pl_vout_raw_flip(gpu.screen.x, gpu.screen.y);
+  else
+    blit();
   gpu.state.fb_dirty = 0;
 }
 
-long GPUopen(void)
+long GPUopen(void **unused)
 {
   gpu.frameskip.active = 0;
   gpu.frameskip.frame_ready = 1;
@@ -124,6 +133,12 @@ void GPUrearmedCallbacks(const struct rearmed_cbs *cbs_)
   gpu.frameskip.advice = &cbs->fskip_advice;
   gpu.frameskip.active = 0;
   gpu.frameskip.frame_ready = 1;
+  gpu.state.hcnt = cbs->gpu_hcnt;
+  gpu.state.frame_count = cbs->gpu_frame_count;
+
+  if (cbs->pl_vout_set_raw_vram)
+    cbs->pl_vout_set_raw_vram(gpu.vram);
+  renderer_set_config(cbs_);
 }
 
 // vim:shiftwidth=2:expandtab
